package com.example;

import org.junit.Test;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class Cn2EnUtils {
    /**
     * 该类为Android测试类，可放置在工程的 test 文件目录下执行
     * 当前我们机构版项目没有test 文件目录，可以自己新建一个工程，然后将该文件放置到test 文件目录下执行
     *
     * 重点-------------------------------------------------------------------------------------------
     * SPLIT_STR 、SOURCE_EN_PATH、REPLACE_CN_PATH需要根据自身环境重新修改过，目前也仅需要修改这三个文件的文件地址
     * -----------------------------------------------------------------------------------------------
     *
     * ios提供过来的数据经分析，分析数据如下
     * 总共查找到的中文源字串数量: 1873
     * 总共查找到的英文源字串数量: 2253
     */
    private static final String SPLIT_STR = "\" = \"";
    // 英文数据源    ios整理的英文字符串源Localizable.strings文件地址
    private static final String SOURCE_EN_PATH = "D:\\strings\\Localizable_en.strings";
    // 中文数据源    ios整理的中文字符串源Localizable.strings文件地址
    private static final String SOURCE_CN_PATH = "D:\\strings\\Localizable.strings";
    // 要替换的目标数据源      自身项目需要将字符串改成英文的文件地址
    private static final String REPLACE_CN_PATH = "D:\\ywzhang.collin\\work_space\\NewWindInfo_dev\\Android.F5\\res\\values-en\\f5_strings.xml";

    @Test
    public void cn2EnReplace() {
        try {
            analysisAndExecuteResult(
                    getAllDataFromSourceCn(),
                    getAllDataFromSourceEn(),
                    getAllDataFromReplaceXml()
            );
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void analysisAndExecuteResult(Map<String, String> cnMap, Map<String, String> enMap
            , ArrayList<WaitReplaceString> replaceStrings) throws IOException {
        System.out.println("--------------------------------------------------");
        System.out.println("开始分析配对字符串");
        for (WaitReplaceString replaceStr : replaceStrings) {
            String replace = replaceStr.waitReplaceStr;
            if (replace == null) {
                continue;
            }
            if (replace.startsWith("\"")) {
                replace = replace.replaceFirst("\"", "");
            }
            if (replace.endsWith("\"")) {
                replace = replace.substring(0, replace.length() - 1);
            }
            String key = cnMap.get(replace);
            String enString = enMap.get(key);
            if (enString == null || enString.length() == 0) {
                enString = enMap.get(replace);
            }
            if (enString == null || enString.length() == 0) {
                continue;
            }
            if(!enString.startsWith("\"")) {
                enString = "\"" + enString;
            }
            if(!enString.endsWith("\"")) {
                enString = enString + "\"";
            }
            System.out.println(replace + " -> " + enString);
            replaceStr.replaceStr = enString;
            replaceStr.newLine = replaceStr.oldLine.replace(
                    replaceStr.waitReplaceStr, replaceStr.replaceStr);
        }

        startReplaceFile(replaceStrings);

    }

    private void startReplaceFile(ArrayList<WaitReplaceString> replaceStrings) throws IOException {
        System.out.println("开始完成配对字符串替换");
        for (WaitReplaceString replaceStr : replaceStrings) {
            if (replaceStr.replaceStr != null) {
                System.out.println(replaceStr.path);
                System.out.println(replaceStr.oldLine);
                System.out.println(replaceStr.newLine);
                System.out.println("-----------------------------------------------");
                File file = new File(replaceStr.path);
                ArrayList<String> oldFileLines = new ArrayList<>();
                if (file.exists() && file.isFile()) {
                    FileReader fileReader = new FileReader(file);
                    BufferedReader reader = new BufferedReader(fileReader);

                    String line;
                    while ((line = reader.readLine()) != null) {
                        if (line.equals(replaceStr.oldLine)) {
                            oldFileLines.add(replaceStr.newLine);
                        } else {
                            oldFileLines.add(line);
                        }
                    }

                    reader.close();
                    fileReader.close();
                }
                if (file.exists() && file.isFile()) {
                    FileWriter fileWriter = new FileWriter(file);
                    BufferedWriter bufferedWriter = new BufferedWriter(fileWriter);

                    for (String line : oldFileLines) {
                        bufferedWriter.write(line);
                        bufferedWriter.newLine();
                    }
                    bufferedWriter.flush();
                    bufferedWriter.close();
                    fileWriter.close();
                }
            }
        }
    }

    private Map<String, String> getAllDataFromSourceEn() throws IOException {
        File file = new File(SOURCE_EN_PATH);
        Map<String, String> sourceMap = new HashMap<>();
        if (file.exists() && file.isFile()) {
            FileReader fileReader = new FileReader(file);
            BufferedReader reader = new BufferedReader(fileReader);
            System.out.println("英文源数据提取");
            String line;
            while ((line = reader.readLine()) != null) {
                String[] splitStrArray = line.split(SPLIT_STR);
                if (splitStrArray.length > 1) {
                    String key = splitStrArray[0];
                    String value = splitStrArray[1];
                    if (key == null || value == null) {
                        continue;
                    }
                    if (key.endsWith(";")) {
                        key = key.substring(0, key.length() - 1);
                    }
                    if (value.endsWith(";")) {
                        value = value.substring(0, value.length() - 1);
                    }
                    sourceMap.put(key.replaceAll("\"", ""),
                            value.replace("\"", ""));
                    System.out.println("key : " + key.replaceAll("\"", "") +
                            " / value : " + value.replace("\"", ""));
                }
            }

            reader.close();
            fileReader.close();
        }
        return sourceMap;
    }

    private Map<String, String> getAllDataFromSourceCn() throws IOException {
        File file = new File(SOURCE_CN_PATH);
        Map<String, String> sourceMap = new HashMap<>();
        if (file.exists() && file.isFile()) {
            FileReader fileReader = new FileReader(file);
            BufferedReader reader = new BufferedReader(fileReader);
            System.out.println("中文源数据提取");
            String line;
            while ((line = reader.readLine()) != null) {
                String[] splitStrArray = line.split(SPLIT_STR);
                if (splitStrArray.length > 1) {
                    String key = splitStrArray[0];
                    String value = splitStrArray[1];
                    if (key == null || value == null) {
                        continue;
                    }
                    if (key.endsWith(";")) {
                        key = key.substring(0, key.length() - 1);
                    }
                    if (value.endsWith(";")) {
                        value = value.substring(0, value.length() - 1);
                    }
                    sourceMap.put(value.replace("\"", ""),
                            key.replace("\"", ""));
                    System.out.println("key : " + key.replaceAll("\"", "") +
                            " / value : " + value.replace("\"", ""));
                }
            }

            reader.close();
            fileReader.close();
        }
        return sourceMap;
    }

    private ArrayList<WaitReplaceString> getAllDataFromReplaceXml() throws IOException {
        return retrievalCNString(new File(REPLACE_CN_PATH));
    }

    private ArrayList<WaitReplaceString> retrievalCNString(File file) {
        ArrayList<WaitReplaceString> waitReplaceStrings = new ArrayList<>();
        try {
            if (file == null || !file.exists() || file.isDirectory()) {
                return waitReplaceStrings;
            }
            FileReader fileReader = new FileReader(file);
            BufferedReader reader = new BufferedReader(fileReader);
            String line;
            String findStr;
            while ((line = reader.readLine()) != null) {
                if ((findStr = containChineseFromXml(line)) != null) {
                    WaitReplaceString replaceStr = new WaitReplaceString();
                    replaceStr.waitReplaceStr = findStr;
                    replaceStr.oldLine = line;
                    replaceStr.path = file.getPath();
                    waitReplaceStrings.add(replaceStr);
                }
            }
            System.out.println("总共需要替换中文字符串数量：" + waitReplaceStrings.size());
            for (WaitReplaceString replaceStr : waitReplaceStrings) {
                System.out.println(replaceStr.waitReplaceStr);
            }
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return waitReplaceStrings;
    }

    private String containChineseFromXml(String content) {
        if (content != null) {
            String pattern1 = ".*[\\u4e00-\\u9faf].*";
            Pattern P1 = Pattern.compile(pattern1);
            // 在匹配尖括号 在匹配出中文
            String pattern3 = "(?<=>(?!/))[^>]*(?=<)";
            Pattern P3 = Pattern.compile(pattern3);
            Matcher matcher3 = P3.matcher(content);
            if (!matcher3.find()) {
                return null;
            }
            String group = matcher3.group();
            Matcher matcher4 = P1.matcher(group);
            if (matcher4.find()) {
                return matcher4.group();
            }
        }
        return null;
    }

    private String containChineseString(String content) {
        if (content != null) {
            String pattern1 = ".*[\\u4e00-\\u9faf].*";
            String pattern2 = "\"([^\"]*)\"";
            // 先匹配出引号 在匹配出中文
            Pattern P2 = Pattern.compile(pattern2);
            Matcher matcher2 = P2.matcher(content);
            if (!matcher2.find()) {
                return null;
            }
            Pattern P1 = Pattern.compile(pattern1);
            Matcher matcher1 = P1.matcher(matcher2.group());
            if (matcher1.find()) {
                return matcher1.group();
            }
        }
        return null;
    }

}
